/*****************************************************************************
*                                                                            *
* void      Adapt_Histo(int BildNr1, int BildNr2, int Fenster)               *
*                                                                            *
******************************************************************************
*                                                                            *
* Adaptive Histogramm Angleichung                                            *
*                                                                            *
* Aus: Computer Vision, Graphics and Image Processing 39, 1987, Page 355-368 *
*      Stephen M. Pizer et. al.                                              *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 10.05.1993                                               *
* Letzte Aenderung: 21.05.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  BildNr1 = Nummer des Ausgangsbildes                                *
*         BildNr2 = Nummer des Zielbildes                                    *
*         Fenster = Groesse der Felder fuer die AHE                          *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define Tr_Matrix(i, j, Grauwert) Tr_Matrix[(long)((i)*16 + (j))*256 + Grauwert]


void Adapt_Histo(int BildNr1, int BildNr2, int Fenster)

{
  long Zmax, Smax;                  /* Zeilen und Spalten des Bildes        */
  unsigned char far *Tr_Matrix;     /* Transformationsmatrix mit den Grauw. */
  int Bloecke;                      /* Anzahl der Bloecke horiz./vertikal   */
  int Grauwert,i,j,k;
  int x, y, z, s;
  long Spalte,Zeile;
  int xr,yr;                        /* Variablen fuer die Randbearbeitung   */
  long Offset_Spalte, Offset_Zeile;
  int X_low,X_high;                 /* Maximaler/minimaler Grauwert im Hist.*/
  float T,a,b;                      /* Hilfsvariablen f. die Interpolation  */
  int X_Mitte, Y_Mitte;             /* Mittelpunktkoordinate eines Blocks   */
  unsigned int Histo[MAX_COLOR];    /* Histogramm eines Blocks              */
  unsigned int Kum_Histo[MAX_COLOR];/* Kumulatives Histogramm               */
  float Abstand_XY[64];             /* Hilfsvektor. Enthaelt Distanzwerte   */
  long Index, Index1, Index2;
  

  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Adaptive Histogrammeinebnung *");

  if (!BildNr1) {
     gotoxy(35,20);
     BildNr1=Read_Int("Quell-Bild ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used)   return;

  if (!BildNr2) {
     gotoxy(35,21);
     BildNr2=Read_Int("Ziel-Bild  ", 2, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE; /* Adresse der beiden Bilder        */
  Index2 = (long) (BildNr2) * PIC_SIZE;

  X_Mitte=Fenster/2;            /* Koordinaten des Blockmittelpunktes       */
  Y_Mitte=Fenster/2;

  Zmax = Bild[BildNr1].Zeilen;
  Smax = Bild[BildNr1].Spalten;

  Tr_Matrix = farmalloc(65536); /* maximal 8*8 Bloecke mit je 256 Grauwert. */

  Bloecke=Smax/Fenster;         /* Anzahl der Bloecke berechnen             */
  for (i=0; i<Fenster; i++)     /* Distanzwerte fuer die Interpolation      */
      Abstand_XY[i]=(float)i / (float)Fenster;

  /*------ Transformations-Funktionen der einzelnen Felder berechnen -------*/
  for (i=0; i<Bloecke; i++) {
      Offset_Zeile=i*Fenster;
      for (j=0; j<Bloecke; j++) {
          Offset_Spalte=j*Fenster;

          /* Histogramm initialisieren                                      */
          for (k=0; k<MAX_COLOR; k++) Histo[k]=0;

          /* Histogramm fuer das aktuelle Feld ermitteln                    */
          for (z=0; z<Fenster; z++) {
              Index = Index1+(long)(Offset_Zeile+z)*Smax;
              for (s=0; s<Fenster; s++)
                  Histo[ Pictures[Index + Offset_Spalte + s] ]++;
          }

          /* Kumulatives Histogramm fuer das Feld aufstellen                */
          Kum_Histo[0]=Histo[0];
          for (k=1; k<MAX_COLOR; k++)
              Kum_Histo[k]=Kum_Histo[k-1]+Histo[k];

          /* Kleinsten vorkommenden Grauwert X_low suchen                   */
          k= -1;
          while( !Histo[++k] );
          X_low=Histo[k];

          /* Groessten vorkommenden Grauwert X_high suchen                  */
          k=MAX_COLOR;
          while( !Histo[--k] );
          X_high=Histo[k];

          T=((float)(Kum_Histo[MAX_COLOR-1]) -
                     0.5*(float)(X_high + X_low));
          if (fabs(T)>EPS_ERROR) T=(float)(WEISS)/T;
          else                   T=0.0;
    
          for (k=0; k<MAX_COLOR; k++)
              Tr_Matrix(i, j, k)=(unsigned char)(T*((float)Kum_Histo[k]-
                                               0.5*(float)(Histo[k]+X_low)));
      }
  }
 
  /* --------------- neue Grauwerte im Inneren berechnen -------------------*/
  for (i=0; i<Bloecke-1; i++) {
      Zeile=Y_Mitte+i*Fenster;
      for (j=0; j<Bloecke-1; j++) {
          Spalte=X_Mitte+j*Fenster;

          for (z=0; z<Fenster; z++) {
              a=Abstand_XY[z];
              for (s=0; s<Fenster; s++) {
                  b=Abstand_XY[s];
                  Grauwert=Pictures[Index1 + (long)(Zeile+z)*Smax + (long)(Spalte+s)];  /* Alter Grauwert */

                  Pictures[Index2 + (long)(Zeile+z)*Smax + (long)(Spalte+s) ]=(unsigned char)
                      (a*(b*(float)Tr_Matrix((i+1), (j+1), Grauwert)+
                         (1.0-b)*(float)Tr_Matrix((i+1), (j  ), Grauwert))+
                      (1.0-a)*(b*(float)Tr_Matrix((i  ), (j+1), Grauwert)+
                         (1.0-b)*(float)Tr_Matrix((i  ), (j  ), Grauwert))
                      );
              }
          }
      }
  }
          
   
  /*--------------------- Ecken und Raender bearbeiten ---------------------*/
  for (i=0; i<Y_Mitte; i++)                           /* linke obere Ecke   */
      for (j=0; j<X_Mitte; j++) {
          Grauwert=Pictures[Index1 + (long)(i)*Smax + (long)(j)];
          Pictures[Index2+(long)i*Smax+(long)j]=
                 Tr_Matrix(0, 0, Grauwert);
      }
  for (i=0; i<Y_Mitte; i++)                           /* rechte obere Ecke  */
      for (j=Smax-X_Mitte; j<Smax; j++) {
          Grauwert=Pictures[Index1 + (long)(i)*Smax + (long)(j)];
          Pictures[Index2+(long)i*Smax+(long)j]=
                 Tr_Matrix(0, Bloecke-1, Grauwert);
      }

  for (i=Zmax-Y_Mitte; i<Zmax; i++)                   /* linke untere Ecke  */
      for (j=0; j<X_Mitte; j++) {
          Grauwert=Pictures[Index1 + (long)(i)*Smax + (long)(j)];
          Pictures[Index2+(long)i*Smax+(long)j]=
                 Tr_Matrix(Bloecke-1, 0, Grauwert);
      }

  for (i=Zmax-Y_Mitte; i<Zmax; i++)                   /* rechte untere Ecke */
      for (j=Smax-X_Mitte; j<Smax; j++) {
          Grauwert=Pictures[Index1 + (long)(i)*Smax + (long)(j)];
          Pictures[Index2+(long)i*Smax+(long)j]=
                 Tr_Matrix(Bloecke-1, Bloecke-1, Grauwert);
      }


 
  /* linker und rechter Rand */
  xr=Smax-X_Mitte;                       /* x-Koordinate des rechten Randes */
  for (i=0; i<Bloecke-1; i++){
     Zeile=X_Mitte+i*Fenster;
     for (y=0; y<Fenster; y++){
         b=Abstand_XY[y];
         for (x=0; x<X_Mitte; x++){
             Grauwert=Pictures[Index1 + (long)(Zeile+y)*Smax + (long)(x)];
             Pictures[Index2 + (long)(Zeile+y)*Smax + (long)(x)]=(unsigned char)
                    (b*Tr_Matrix(i+1, 0, Grauwert)+(1.0-b)*Tr_Matrix(i, 0, Grauwert));
             Grauwert=Pictures[Index1 + (long)(Zeile+y)*Smax + (long)(xr+x)];
             Pictures[Index2 + (long)(Zeile+y)*Smax + (long)(xr+x)]=(unsigned char)
                    (b*Tr_Matrix(i+1, Bloecke-1, Grauwert)+(1.0-b)*Tr_Matrix(i, Bloecke-1, Grauwert));
        }
     }
  }

  /* oberer und unterer Rand */
  yr=Zmax-X_Mitte;                       /* y-Koordinate des unteren Randes */
  for (i=0; i<Bloecke-1; i++) {
      Spalte=X_Mitte+i*Fenster;
      for (x=0; x<Fenster; x++) {
          a=Abstand_XY[x];
          for (y=0; y<X_Mitte; y++) {
              Grauwert=Pictures[Index1 + (long)(y)*Smax + (long)(Spalte+x)];
              Pictures[Index2 + (long)(y)*Smax + (long)(Spalte+x)]=(unsigned char)
                     (a*Tr_Matrix(0, i+1, Grauwert)+(1.0-a)*Tr_Matrix(0, i, Grauwert));
              Grauwert=Pictures[Index1 + (long)(yr+y)*Smax + (long)(Spalte+x)];
              Pictures[Index2 + (long)(yr+y)*Smax + (long)(Spalte+x)]=(unsigned char)
                     (a*Tr_Matrix(Bloecke-1, i+1, Grauwert)+(1.0-a)*Tr_Matrix(Bloecke-1, i, Grauwert));
        }
     }
  }

  farfree(Tr_Matrix);                /* belegter Speicher wieder frei geben */

  /* Bildinformation zum Schluss korrekt setzen                             */
  Bild[BildNr2].Zeilen =Zmax;
  Bild[BildNr2].Spalten=Smax;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

}
