/*****************************************************************************
*                                                                            *
* void      Grauwertmanipu(int Art, int BildNr, float Faktor_Schwelle)       *
*                                                                            *
******************************************************************************
*                                                                            *
* Aendert den Grauwert in einem Bild nach verschiedenen Verfahren            *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 02.10.1992                                               *
* Letzte Aenderung: 10.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  Art = 1 : Grauwertverschiebung                                     *
*             = 2 : Kontrastaenderung                                        *
*             = 3 : Grauwertquantisierung                                    *
*             = 4 : Bild invertieren                                         *
*             = 5 : Binaerbild erzeugen                                      *
*         BildNr = Nummer des Ausgangs-/Zielbildes (1<=BildNr<=MAX_PICTURE)  *
*         Faktor_Schwelle = Faktor/Schwelle zur Grauwqertmanipulation        *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(), Binaerbild()       *
*             Read_Char(), Upper(), Make_Icon(), Show_Pic()                  *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Grauwertmanipu(int Art, int BildNr, float Faktor_Schwelle)
{
  unsigned char Grauwert[MAX_COLOR];              /* Transformations-LUT    */
  long s;                                         /* Zeilen-/Spaltenzaehler */
  int Pixel, Offset, Schwelle, k;
  long Index;                                     /* Bild-Anfangsadresse    */
  float Step, Stufe;
  float Faktor;
  char Modulo;                                    /* Flag fuer Modulo-Oper. */

  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Grauwertmanipulation *");

  if (BildNr == FALSE) {
     gotoxy(35,20);
     BildNr=Read_Int("Bild Nr. ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr<=0) || (BildNr>MAX_PICTURE)) return;
  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr].Used) return;

  Index = (long) (BildNr) * PIC_SIZE;

  switch (Art) {
     case 1:
         /*----------------- Art = 1 : Grauwertverschiebung ----------------*/
         if (Faktor_Schwelle == FALSE) {
            gotoxy(35,22);
            Offset =Read_Int("Verschiebungsgroesse", INT_NO_DEFAULT, ':');
            gotoxy(35,23);
            Modulo=Upper(Read_Char("Verschiebung Modulo (J/N)", 'N'));
         }
         else {
            Offset=(int)Faktor_Schwelle;
            Modulo='N';
         }
         for (s=0; s<PIC_SIZE; s++) {
             Pixel = Offset + Pictures[Index];
             if (Modulo=='J')
                 Pictures[Index++] = (unsigned char) Pixel;
             else
                 Pictures[Index++] =
                    (Pixel>WEISS) ? WEISS : (Pixel<SCHWARZ) ? SCHWARZ : Pixel;
         }
         break;
     case 2:
         /*----------------- Art = 2 : Kontrastaenderung -------------------*/
         if (Faktor_Schwelle == FALSE) {
            gotoxy(35,22);
            Faktor=Read_Float("Kontrastaenderungsfaktor", FLOAT_NO_DEFAULT, ':');
            gotoxy(35,23);
            Modulo=Upper(Read_Char("Aenderung Modulo (J/N)", 'N'));
         }
         else {
            Faktor=Faktor_Schwelle;
            Modulo='N';
         }
         for (s=0; s<PIC_SIZE; s++) {
             Pixel = (int) (Faktor * (float)Pictures[Index]);
             if (Modulo=='J')
                 Pictures[Index++] = (unsigned char) Pixel;
             else
                 Pictures[Index++] =
                    (Pixel>WEISS) ? WEISS : (Pixel<SCHWARZ) ? SCHWARZ : Pixel;
         }
         break;
     case 3:
         /*----------------- Art = 3 : Grauwertquantisierung ---------------*/
         if (Faktor_Schwelle == FALSE) {
            do {
               gotoxy(35,22);
               Schwelle = Read_Int("Anzahl der Graustufen (2-256) ",
                                    INT_NO_DEFAULT, ':' );
            } while ((Schwelle < 1) || (Schwelle >MAX_COLOR));
            if (!Schwelle) return;                          /* ==> Abbruch! */
         }
         else {
            Schwelle=(int)Faktor_Schwelle;
         }
         Step  = (float)MAX_COLOR / (float)Schwelle;        /* Schrittweite */
         Stufe = Step-0.5;                       /* aktuelle Grauwert-Stufe */

         /* Neue "Look-Up-Tabelle" erstellen ...                            */
         for (k=0; k<MAX_COLOR; k++)
             if ((float)k <= Stufe)
                Grauwert[k]=(int)Stufe;
             else {
                Stufe += Step;
                Grauwert[k]=(int)Stufe;
             }

         /* ... und Bild aufgrund dieser LUT transformieren.                */
         for (s=0; s<PIC_SIZE; s++)
             Pictures[Index] = Grauwert[ Pictures[Index++] ];
         break;
     case 4:
         /*----------------- Art = 4 : Bild invertieren --------------------*/
         for (s=0; s<PIC_SIZE; s++)
             Pictures[Index]=MAX_COLOR-Pictures[Index++]-1;
         break;
     case 5:
         /*----------------- Art = 5 : Binaerbild erzeugen -----------------*/
         gotoxy(35,22);
         Schwelle=Read_Int("Binaer-Schwelle ", 128, ':');
         Binaerbild(BildNr+1, Schwelle);
         break;
  }

  Make_Icon(BildNr);
  Show_Pic(BildNr, BildNr+1);

}
