/*****************************************************************************
*                                                                            *
* void      Prewitt(int BildNr1, int BildNr2)                                *
*                                                                            *
******************************************************************************
*                                                                            *
* Wendet den Prewitt-Operator auf ein Grauwertbild und und zeigt das         *
* Gradientenbild, skaliert auf den Bereich 0-255, an.                        *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 24.10.1992                                               *
* Letzte Aenderung: 24.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr1 = Quelbild                                                  *
*        BildNr2 = Zielbild (Gradientenbild)                                 *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Skalier_Histo(), Make_Icon(), Show_Pic()                       *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define SKALIERUNG 2.0


void Prewitt(int BildNr1, int BildNr2)
{
  long  z, s, Z_Temp, S_Temp;     /* Variablen fuer Zeilen und Spaltenindex */
  long  Index1, Index2, Index;    /* Adresse der beiden Bilder              */
  long  zwerg1, zwerg2;
  long  Zmax, Smax;


  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Anwendung des Prewitt-Operators *");

  if (!BildNr1) {
     gotoxy(35,20);
     BildNr1=Read_Int("Quell-Bild      ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used)   return;

  if (!BildNr2) {
     gotoxy(35,21);
     BildNr2=Read_Int("Gradienten-Bild ", 2, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE; /* Adresse der beiden Bilder        */
  Index2 = (long) (BildNr2) * PIC_SIZE;

  Zmax = Bild[BildNr1].Zeilen;
  Smax = Bild[BildNr1].Spalten;

  /*------------ Faltung durchfuehren ------------------*/
  /*            1  1  1                1  0 -1          */
  /*   Maske1:  0  0  0      Maske2:   1  0 -1          */
  /*           -1 -1 -1                1  0 -1          */
  /*----------------------------------------------------*/

  for (z=0; z<Zmax; z++) {
      for (s=0; s<Smax; s++) {
          /*---- erste Zeile ----*/
          Z_Temp = (z>0) ? (z-1) : 0;
          Index=Index1+Z_Temp*Smax;
          S_Temp = (s>0) ? (s-1) : 0;
          zwerg1 = zwerg2 = Pictures[Index+S_Temp];
          zwerg1 += Pictures[Index+s];
          S_Temp = (s<(Smax-1)) ? (s+1) : (Smax-1);
          Index=Index+S_Temp;
          zwerg1 += Pictures[Index];
          zwerg2 -= Pictures[Index];

          /*---- zweite Zeile ----*/
          Index=Index1+z*Smax;
          S_Temp = (s>0) ? (s-1) : 0;
          zwerg2 += Pictures[Index+S_Temp];
          S_Temp = (s<(Smax-1)) ? (s+1) : (Smax-1);
          zwerg2 -= Pictures[Index+S_Temp];

          /*---- dritte Zeile ----*/
          Z_Temp = (z<(Zmax-1)) ? (z+1) : (Zmax-1);
          Index=Index1+Z_Temp*Smax;
          S_Temp = (s>0) ? (s-1) : 0;
          zwerg1 -= Pictures[Index+S_Temp];
          zwerg2 += Pictures[Index+S_Temp];
          zwerg1 -= Pictures[Index+s];
          S_Temp = (s<(Smax-1)) ? (s+1) : (Smax-1);
          Index=Index+S_Temp;
          zwerg1 -= Pictures[Index];
          zwerg2 -= Pictures[Index];

          /*---- Vektorielle Addition ----*/
          zwerg1=(int)( sqrt( (double)(zwerg1*zwerg1 + zwerg2*zwerg2) )/SKALIERUNG );
          Pictures[Index2++]=(zwerg1>WEISS) ? WEISS : zwerg1;
      }
  }

  /* Bildinformation zum Schluss korrekt setzen                             */
  Bild[BildNr2].Zeilen =Zmax;
  Bild[BildNr2].Spalten=Smax;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  /*------- Ausgabe des Gradientenbildes mit 255 als Obergrenze (5%) -------*/
  Skalier_Histo(BildNr2+1, 5);

  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);
}
