/*****************************************************************************
*                                                                            *
* void      Skalier_Histo(int BildNr, int Prozent)                           *
*                                                                            *
******************************************************************************
*                                                                            *
* Fuehrt eine Grauwert-Intensitaetsskalierung durch. Die oberen Prozent%     *
* Punkte werden dabei auf den Grauwert (100-Prozent)*255 gesetzt und der     *
* Rest dann auf den Gesamtbereich skaliert.                                  *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 24.10.1992                                               *
* Letzte Aenderung: 25.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  BildNr = Nummer des zu bearbeitenden Bildes                        *
*         Prozent= obere Prozentsatz der abgeschnitten werden soll           *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von:                                                                *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Skalier_Histo(int BildNr, int Prozent)
{
  unsigned int  Grau[MAX_COLOR];    /* Haeufigkeit der einzelnen Grauwerte  */
  long Index, Index1;               /* Adresse des Bildes                   */
  long         z;
  long GesamtPixel, Anzahl;         /* kleinster/groesster vorkommender Grw.*/
  float Faktor;                     /* Skalierungsfaktor                    */
  int          i;


  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */
  if ( (BildNr<0) || (BildNr>MAX_PICTURE) || (!Bild[BildNr].Used) ||
       (Prozent<=0) || (Prozent>=100) )     return;

  /* Initialisierung aller Arrays und Variablen                             */
  for (i=0; i<MAX_COLOR; Grau[i++]=0);

  Index  = (long) (BildNr) * PIC_SIZE;  /* Adresse des Quell-/Zielbildes    */
  Index1 = Index;

  /* Haeufigkeit der einzelnen Grauwert ermitteln                           */
  GesamtPixel = (long)Bild[BildNr].Zeilen * (long)Bild[BildNr].Spalten;
  for (z=0; z<GesamtPixel; z++)
      Grau[Pictures[Index1++]]++;

  /* Den groessten vorkommenden Grauwert suchen                             */
  z=MAX_COLOR;
  Anzahl=Grau[--z];
  while ( ((Anzahl*100/GesamtPixel) < Prozent) && (z>2) )
        Anzahl += Grau[--z];

  Faktor = (float) (MAX_COLOR-1) / (float) (z);

  for (i=0; i<z;         i++) Grau[i] = (int) ((float) i * Faktor);
  for (i=z; i<MAX_COLOR; i++) Grau[i] = WEISS;

  /* Zum Schluss die eigentliche Umsetzung der Grauwert vornehmen           */
  Index1=Index;
  for (z=0; z<GesamtPixel; z++)
      Pictures[Index1]=Grau[ Pictures[Index1++] ];

}
