/*****************************************************************************
*                                                                            *
* void      Textur_Coocc(int BildNr1, int BildNr2)                           *
*                                                                            *
******************************************************************************
*                                                                            *
* Berechnet vier Cooccurrenz-Matrizen, d.h. Grauwert-Uebergangsmatrizen.     *
* Die Bilder sind dazu auf 128 Graustufen quantisiert. Es werden dann        *
* folgende Uebergaenge verwendet:   a b    a      a     a                    *
*                                           b     b    b                     *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 04.03.1993                                               *
* Letzte Aenderung: 04.03.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr1 = Quelbild                                                  *
*        BildNr2 = Zielbild (Cooccurrenz-Matrix)                             *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Grauwertmanipu(), Make_Icon(), Show_Pic()                      *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Textur_Coocc(int BildNr1, int BildNr2)
{
  long  z, s;                     /* Variablen fuer Zeilen und Spaltenindex */
  long  Index1, Index2, Index;    /* Adresse der beiden Bilder              */
  long  Index_Tmp;
  long  Offset1, Offset2, Offset3;
  long  Smax, Zmax, Wert_a, Wert_b;


  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Berechnung der Cooccurrenz-Matrix *");

  if (!BildNr1) {
     gotoxy(35,20);
     BildNr1=Read_Int("Quell-Bild       ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used)   return;

  if (!BildNr2) {
     gotoxy(35,21);
     BildNr2=Read_Int("Cooccurrenz-Bild ", 2, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  /* Zuerst Grauwertquantisierung auf 128 Stufen. Daher kann das Bild in 2x2*/
  /* Felder fuer die Ergebnisse der Cooccurrenz-Matrizen aufgeteilt werden. */
  Grauwertmanipu( 3, BildNr1+1, 128);

  ClearTextWindow(35,18,80,19);
  WriteText(35,18,"* Berechnung der Cooccurrenz-Matrix *");

  /* Anmerkung: ggf. Delta_X und Delta_Y einlesen (hier: jeweils = 1)       */

  Index1 = (long) (BildNr1) * PIC_SIZE; /* Adresse der beiden Bilder        */
  Index2 = (long) (BildNr2) * PIC_SIZE;

  Zmax = Bild[BildNr1].Zeilen;
  Smax = Bild[BildNr1].Spalten;

  for (z=0; z<PIC_SIZE; z++) Pictures[Index2 + z]=SCHWARZ;

  Offset1 = S_RES_MAX>>1;                   /* Offset fuer die 4 Teilbilder */
  Offset2 = Offset1*S_RES_MAX;
  Offset3 = Offset1+Offset2;

  for (z=1; z<(Zmax-1); z++) {
      Index = Index1 + z*Smax;
      for (s=1; s<(Smax-1); s++) {
          Wert_a = (long) (Pictures[Index+s]   >> 1);       /* rechts neben */
          Wert_b = (long) (Pictures[Index+s+1] >> 1);

          Index_Tmp=Index2 + Wert_b*S_RES_MAX + Wert_a;
          if (Pictures[Index_Tmp]<WEISS) Pictures[Index_Tmp]++;

          Wert_b = (long) (Pictures[Index+s+1+Smax] >> 1);  /* rechts unten */
          Index_Tmp=Index2 + Wert_b*S_RES_MAX + Wert_a + Offset1;
          if (Pictures[Index_Tmp]<WEISS) Pictures[Index_Tmp]++;

          Wert_b = (long) (Pictures[Index+s+Smax] >> 1);    /* unten        */
          Index_Tmp=Index2 + Wert_b*S_RES_MAX + Wert_a + Offset2;
          if (Pictures[Index_Tmp]<WEISS) Pictures[Index_Tmp]++;

          Wert_b = (long) (Pictures[Index+s-1+Smax] >> 1);  /* links unten  */
          Index_Tmp=Index2 + Wert_b*S_RES_MAX + Wert_a + Offset3;
          if (Pictures[Index_Tmp]<WEISS) Pictures[Index_Tmp]++;
      }
  }

  /* Trennkreuz fuer die 4 Felder einzeichnen                               */
  for (s=0; s<S_RES_MAX; s++) Pictures[Index2+Offset2+s]          =WEISS;
  for (z=0; z<Z_RES_MAX; z++) Pictures[Index2+Offset1+S_RES_MAX*z]=WEISS;

  /* Bildinformation zum Schluss korrekt setzen                             */
  Bild[BildNr2].Zeilen =Z_RES_MAX;
  Bild[BildNr2].Spalten=S_RES_MAX;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

}
